<!---
	Name         : C:\projects\lighthousepro\wwwroot\lighthousepro\components\IssueManager.cfc
	Author       : Raymond Camden 
	Created      : 11/02/05
	Last Updated : 12/30/05
	History      : Changed relatedURL in sql to relatedurl (rkc 11/2/05)
				   Added printFormat/paragraphformat2 (rkc 12/22/05)
				   Added support for mail user and attachment (rkc 12/28/05)
				   Fixed problem with email that went out (rkc 12/30/05)
--->

<cfcomponent output="false" displayName="Issue Manager">

	<cfset variables.dsn = "">
	<cfset variables.emailfrom = "">
	<cfset variables.mailusername = "">
	<cfset variables.mailpassword = "">
	<cfset variables.attachmentpath = "">
	
	<cffunction name="init" access="public" returnType="IssueManager" output="false">
		<cfargument name="dsn" type="string" required="true">
		<cfargument name="emailFrom" type="string" required="true">
		<cfargument name="mailusername" type="string" required="true">
		<cfargument name="mailpassword" type="string" required="true">
		<cfargument name="attachmentpath" type="string" required="true">
		
		<cfset variables.dsn = arguments.dsn>
		<cfset variables.emailfrom = arguments.emailfrom>
		<cfset variables.mailusername = arguments.mailusername>
		<cfset variables.mailpassword = arguments.mailpassword>
		<cfset variables.attachmentpath = arguments.attachmentpath>
		
		<cfset variables.projectManager = createObject("component","ProjectManager").init(variables.dsn)>
		<cfset variables.projectDAO = createObject("component","ProjectDAO").init(variables.dsn)>
		<cfset variables.issueDAO = createObject("component","IssueDAO").init(variables.dsn, variables.attachmentpath)>
		<cfset variables.userDAO = createObject("component","UserDAO").init(variables.dsn)>
		<cfset variables.lociDAO = createObject("component","ProjectLocusDAO").init(variables.dsn)>
		
		<cfreturn this>
	</cffunction>	
		
	<cffunction name="emailNotifications" access="public" returnType="void" output="false"
				hint="Sends an email to those subscribed to the projects.">
		<cfargument name="issueid" type="uuid" required="true">
		<cfargument name="oldBean" type="IssueBean" required="false">
		<cfset var issueBean = variables.issueDAO.read(arguments.issueid)>
		<cfset var projectid = issueBean.getProjectIDFK()>
		<cfset var emails = "">
		<cfset var project = "">
		<cfset var user = "">
		<cfset var olduser = "">
		<cfset var locus = "">
		<cfset var oldlocus = "">
		<cfset var status = getStatus(issueBean.getStatusIDFK())>
		<cfset var oldstatus = "">
		<cfset var severity = getSeverity(issueBean.getSeverityIDFK())>
		<cfset var oldseverity = "">
		<cfset var mailbody = "">
		
		<cfinvoke component="#variables.ProjectManager#" method="getSubscribedUsersForProject" id="#projectid#" returnVariable="emails">
		<cfinvoke component="#variables.ProjectDAO#" method="read" id="#projectid#" returnVariable="project">
		<cfinvoke component="#variables.UserDAO#" method="read" id="#issueBean.getUserIDFK()#" returnVariable="user">
		<cfinvoke component="#variables.LociDAO#" method="read" id="#issueBean.getLocusIDFK()#" returnVariable="locus">
		
		<cfif isDefined("arguments.oldBean")>
			<cfinvoke component="#variables.LociDAO#" method="read" id="#arguments.oldBean.getLocusIDFK()#" returnVariable="oldlocus">
			<cfinvoke component="#variables.UserDAO#" method="read" id="#arguments.oldBean.getUserIDFK()#" returnVariable="olduser">
			<cfset oldstatus = getStatus(arguments.oldBean.getStatusIDFK())>
			<cfset oldseverity = getSeverity(arguments.oldBean.getSeverityIDFK())>
		</cfif>

		<cfsavecontent variable="mailbody">
		<cfoutput>
		<pre>
Lightouse Pro Notification
<p></p>
<strong>Tracking ID:</strong>		#issueBean.getID()#
<strong>Project:</strong>		#project.getname()#
<strong>Issue:</strong>			#issueBean.getName()# <cfif isDefined("arguments.oldBean") and issueBean.getName() neq arguments.oldBean.getName()> (was #arguments.oldBean.getName()#)</cfif> 
<strong>Locus:</strong>			#locus.getName()# <cfif isDefined("arguments.oldBean") and locus.getName() neq oldLocus.getName()> (was #oldLocus.getName()#)</cfif>
<strong>Severity:</strong>		#severity# <cfif isDefined("arguments.oldBean") and severity neq oldseverity> (was #oldseverity#)</cfif>
<strong>Status:</strong>			#status# <cfif isDefined("arguments.oldBean") and status neq oldstatus> (was #oldstatus#)</cfif>
<strong>Type:</strong>			<cfif issueBean.getIsBug()>Bug<cfelse>Enhancement</cfif> <cfif isDefined("arguments.oldBean") and issueBean.getIsBug() neq arguments.oldBean.getIsBug()> (was <cfif arguments.oldBean.getIsBug()>Bug<cfelse>Enhancement</cfif>)</cfif>
<strong>Related URL:</strong>	#issueBean.getRelatedURL()#
<strong>Created:</strong>		#dateFormat(issueBean.getCreated(),"m/d/yy")# #timeFormat(issueBean.getCreated(),"h:mm tt")#
<strong>Updated:</strong>		#dateFormat(issueBean.getUpdated(),"m/d/yy")# #timeFormat(issueBean.getUpdated(),"h:mm tt")#
<strong>Owner:</strong>			#user.getName()# <cfif isDefined("arguments.oldBean") and user.getName() neq oldUser.getName()> (was #oldUser.getName()#)</cfif>

<strong>Description:</strong>
#issueBean.getDescription()#

<strong>History:</strong>
#issueBean.getHistory()#
</pre>					
		</cfoutput>
		</cfsavecontent>
		
		<cfif variables.mailusername is not "" and variables.mailpassword is not "">
			<cfmail query="emails" to="#emailaddress#" from="#variables.emailfrom#" subject="Issue Tracking Number #issueBean.getID()#" type="HTML" username="#variables.mailusername#" password="#variables.mailpassword#">#mailbody#</cfmail>
		<cfelse>
			<cfmail query="emails" to="#emailaddress#" from="#variables.emailfrom#" subject="Issue Tracking Number #issueBean.getID()#" type="HTML">#mailbody#</cfmail>
		</cfif>
			
	</cffunction>
		
	<cffunction name="getIssues" access="public" returnType="query" output="false"
				hint="Gets all the issues. Allows for various filtering ops.">		
		<cfargument name="sort" type="string" required="false" default="updated">
		<cfargument name="sortdir" type="string" required="false" default="desc">
		<cfargument name="projectidfk" type="string" required="false" default="">
		<cfargument name="projectidfks" type="string" required="false" default="">
		<cfargument name="useridfk" type="string" required="false" default="">
		
		<cfset var data = "">

		<cfquery name="data" datasource="#variables.dsn#">
			select 		issues.id, issues.projectidfk, issues.created, issues.updated, 
						issues.name, issues.useridfk, issues.history, 
						issues.isbug, issues.locusidfk, issues.severityidfk, issues.statusidfk, issues.relatedurl, issues.attachment,
						projectloci.name as locusname,
						severities.name as severityname,
						statuses.name as statusname,
						projects.name as projectname,
						users.name as username
			<cfif application.dbtype eq "oracle">
						,to_char(dbms_lob.substr( issues.description, 32765, 1 ))  as description		
			from		issues,
						projectloci,
						severities,
						statuses,
						projects,
						users
			where		1=1
			And			issues.locusidfk = projectloci.id(+)
			And			issues.severityidfk = severities.id(+)
			And			issues.statusidfk = statuses.id(+)
			And			issues.projectidfk = projects.id(+)
			And			issues.useridfk = users.id(+)
			<cfelse>	
			,issues.description
			from		((((issues left join projectloci on issues.locusidfk = projectloci.id)
						left join severities on issues.severityidfk = severities.id)
						left join statuses on issues.statusidfk = statuses.id)
						left join projects on issues.projectidfk = projects.id)
						left join users on issues.useridfk = users.id
			where		1=1					
			</cfif>			
			<cfif arguments.projectidfk neq "">
			and			projectidfk = <cfqueryparam cfsqltype="cf_sql_varchar" value="#arguments.projectidfk#" maxlength="35">
			</cfif>
			<cfif arguments.projectidfks neq "">
			and			projectidfk in (<cfqueryparam cfsqltype="cf_sql_varchar" value="#arguments.projectidfks#" list=true>)
			</cfif>
			<cfif arguments.useridfk neq "">
			and			useridfk = <cfqueryparam cfsqltype="cf_sql_varchar" value="#arguments.useridfk#" maxlength="35">
			</cfif>

			order by 	#arguments.sort# #arguments.sortdir#
		</cfquery>

		<cfreturn data>
	</cffunction>
	
	<cffunction name="getIssuesForProject" access="public" returnType="query" output="false"
				hint="Filter method to get issues for one project.">
		<cfargument name="projectidfk" type="uuid" required="true">
		
		<cfreturn getIssues(projectidfk=arguments.projectidfk)>
	</cffunction>

	<cffunction name="getIssuesForProjects" access="public" returnType="query" output="false"
				hint="Filter method to get issues for multiple projects.">
		<cfargument name="projectidfks" type="string" required="true">
		
		<cfreturn getIssues(projectidfks=arguments.projectidfks)>
	</cffunction>

	<cffunction name="getIssuesForUserID" access="public" returnType="query" output="false"
				hint="Filter method to get issues for one user.">
		<cfargument name="useridfk" type="uuid" required="true">
		
		<cfreturn getIssues(useridfk=arguments.useridfk)>
	</cffunction>
	
	<cffunction name="getSeverities" access="public" returnType="query" output="false"
				hint="Gets possible severity values for issues.">
		<cfset var data = "">
		
		<cfquery name="data" datasource="#variables.dsn#">
			select 		id, name
			from		severities
			order by 	rank asc
		</cfquery>
		
		<cfreturn data>
		
	</cffunction>

	<cffunction name="getSeverity" access="public" returnType="string" output="false"
				hint="Gets a severity name.">
		<cfargument name="id" type="uuid" required="true">
		<cfset var data = "">
		
		<cfquery name="data" datasource="#variables.dsn#">
			select 		name
			from		severities
			where		id = <cfqueryparam cfsqltype="cf_sql_varchar" value="#arguments.id#" maxlength="35">
		</cfquery>
		
		<cfreturn data.name>
		
	</cffunction>

	<cffunction name="getStatus" access="public" returnType="string" output="false"
				hint="Gets a status name.">
		<cfargument name="id" type="uuid" required="true">
		<cfset var data = "">
		
		<cfquery name="data" datasource="#variables.dsn#">
			select 		name
			from		statuses
			where		id = <cfqueryparam cfsqltype="cf_sql_varchar" value="#arguments.id#" maxlength="35">
		</cfquery>
		
		<cfreturn data.name>
		
	</cffunction>

	<cffunction name="getStatuses" access="public" returnType="query" output="false"
				hint="Gets possible status values for issues.">
		<cfset var data = "">
		
		<cfquery name="data" datasource="#variables.dsn#">
			select 		id, name
			from		statuses
			order by	rank asc
		</cfquery>
		
		<cfreturn data>
		
	</cffunction>

	<!--- This is a copy of the UDF from udfs.cfm. This is bad. But this is the only CFC using it, so I feel ok about it. Later on I'll switch to a utils.cfc --->
	<cffunction name="paragraphFormat2" returnType="string" access="private">
		<cfargument name="str" type="string" required="true">
		
		<cfset arguments.str = replace(str,chr(13)&chr(10),chr(10),"ALL")>
		<cfset arguments.str = replace(str,chr(13),chr(10),"ALL")>
		<cfset arguments.str = replace(str,chr(9),"&nbsp;&nbsp;&nbsp;","ALL")>
		<cfreturn replace(str,chr(10),"<br />","ALL")>

	</cffunction>
	
	<cffunction name="printFormat" returnType="string" access="public"
				hint="Used to render the issue for passing to PDF, or whatever else you may want.">
		<cfargument name="i" type="IssueBean" required="true">
		<cfset var result = "">
		<cfset var p = variables.projectDAO.read(arguments.i.getProjectIDFK())>
		<cfset var locus = variables.lociDAO.read(arguments.i.getLocusIDFK())>
		<cfset var user = variables.userDAO.read(arguments.i.getUserIDFK())>
		
		<cfsavecontent variable="result">
		<cfoutput>
<html>

<head>
</head>

<body>
<p>
<table width="90%">
	<tr>
		<td><b>Tracking ID:</b></td>
		<td>#arguments.i.getID()#</b></td>
	</tr>
	<tr>
		<td><b>Name:</b></td>
		<td>#arguments.i.getName()#</b></td>
	</tr>
	<tr>
		<td><b>Project:</b></td>
		<td>#p.getName()#</td>
	</tr>
	<tr>
		<td><b>Type:</b></td>
		<td><cfif arguments.i.getIsBug()>Bug<cfelse>Enhancement</cfif></td>
	</tr>
	<tr>
		<td><b>Locus:</b></td>
		<td>#locus.getName()#</td>
	</tr>
	<tr>
		<td><b>Severity:</b></td>
		<td>#getSeverity(arguments.i.getSeverityIDFK())#</td>
	</tr>
	<tr>
		<td><b>Status:</b></td>
		<td>#getStatus(arguments.i.getStatusIDFK())#</td>
	</tr>

	<tr>
		<td><b>Related URL:</b></td>
		<td>#arguments.i.getRelatedURL()#</td>
	</tr>
	<tr>
		<td><b>Owner:</b></td>
		<td>#user.getName()# (#user.getUsername()#)</td>
	</tr>
	<tr>
		<td><b>Created:</b></td>
		<td>#dateFormat(arguments.i.getCreated(),"mm/dd/yy")# #timeFormat(arguments.i.getCreated(),"h:mm tt")#</td>
	</tr>
	<tr>
		<td><b>Last Updated:</b></td>
		<td>#dateFormat(arguments.i.getUpdated(),"mm/dd/yy")# #timeFormat(arguments.i.getUpdated(),"h:mm tt")#</td>
	</tr>
	<tr>
		<td colspan="2">&nbsp;</td>
	</tr>
	<tr>
		<td colspan="2"><b>Description:</b><br>
		#paragraphFormat(arguments.i.getDescription())#</td>
	</tr>
	<tr>
		<td colspan="2">&nbsp;</td>
	</tr>

	<tr>
		<td colspan="2"><b>History:</b><br>
		#paragraphFormat2(arguments.i.getHistory())#</td>
	</tr>

</table>
</p>

</body>
</html>
		</cfoutput>
		</cfsavecontent>
	
		<cfreturn trim(result)>
	</cffunction>
	
			
</cfcomponent>