<!---
	Name         : C:\projects\lighthousepro\wwwroot\lighthousepro\components\IssueManager.cfc
	Author       : Raymond Camden 
	Created      : Sometime in the past - before there were computers...
	Last Updated : 1/11/06
	History      : various 2.0 mods (rkc 12/22/05)
				 : sort getProjectsByUser (rkc 12/29/05)
				 : added a new function get loci for a list of projects (rkc 1/11/06)
--->

<cfcomponent output="false" displayName="Project Manager">

	<cfset variables.dsn = "">
	
	<cffunction name="init" access="public" returnType="ProjectManager" output="false">
		<cfargument name="dsn" type="string" required="true">
		
		<cfset variables.dsn = arguments.dsn>
		<cfset variables.projectDAO = createObject("component","ProjectDAO").init(variables.dsn)>
		<cfreturn this>
	</cffunction>	
	
	<cffunction name="clearLociForProject" access="public" returnType="void" output="false"
				hint="Clears all loci for a project.">		
		<cfargument name="id" type="uuid" required="true" hint="Project ID">
		
		<cfquery name="data" datasource="#variables.dsn#">
		delete from projects_projectloci
		where projectidfk = <cfqueryparam cfsqltype="cf_sql_varchar" value="#arguments.id#" maxlength="35">
		</cfquery>
		
	</cffunction>

	<cffunction name="clearEmailProjectsForUser" access="public" returnType="void" output="false"
				hint="Removes the user from all email subscriptions.">
		<cfargument name="userid" type="uuid" required="true">		

		<cfquery datasource="#variables.dsn#">
		delete from projects_users_email
		where useridfk = <cfqueryparam cfsqltype="cf_sql_varchar" value="#arguments.userid#" maxlength="35">
		</cfquery>
			
	</cffunction>

	<cffunction name="clearProjectsForUser" access="public" returnType="void" output="false"
				hint="Clears all projects for a user.">		
		<cfargument name="id" type="uuid" required="true" hint="Project ID">
		
		<cfquery name="data" datasource="#variables.dsn#">
		delete from projects_users
		where useridfk = <cfqueryparam cfsqltype="cf_sql_varchar" value="#arguments.id#" maxlength="35">
		</cfquery>
		
	</cffunction>

	<cffunction name="clearUsersForProject" access="public" returnType="void" output="false"
				hint="Clears all users for a project.">		
		<cfargument name="id" type="uuid" required="true" hint="Project ID">
		
		<cfquery name="data" datasource="#variables.dsn#">
		delete from projects_users
		where projectidfk = <cfqueryparam cfsqltype="cf_sql_varchar" value="#arguments.id#" maxlength="35">
		</cfquery>
		
	</cffunction>

	<cffunction name="getEmailProjectsForUser" access="public" returnType="query" output="false"
				hint="Gets all the projects a user wants email for.">
		<cfargument name="username" type="string" required="false" default="#getAuthUser()#">		
		<cfset var data = "">
		
		<cfquery name="data" datasource="#variables.dsn#">
			select 	projects.id, projects.name
			from	projects, projects_users_email, users
			where 	projects.id = projects_users_email.projectidfk
			and   	projects_users_email.useridfk = users.id
			and	  	users.username = <cfqueryparam cfsqltype="cf_sql_varchar" value="#arguments.username#" maxlength="35">
		</cfquery>

		<cfreturn data>
	</cffunction>
	
	<cffunction name="getLociForProject" access="public" returnType="query" output="false"
				hint="Gets loci for one project.">		
		<cfargument name="id" type="uuid" required="true" hint="Project ID">

		<cfreturn getLociForProjects(arguments.id)>
	</cffunction>

	<cffunction name="getLociForProjects" access="public" returnType="query" output="false"
				hint="Gets loci for one project.">		
		<cfargument name="projectidlist" type="string" required="true" hint="Project ID">
		<cfset var data = "">
		
		<cfquery name="data" datasource="#variables.dsn#">
			select	projectloci.id, projectloci.name
			from	projects_projectloci, projectloci
			where	projects_projectloci.projectidfk in (<cfqueryparam cfsqltype="cf_sql_varchar" value="#arguments.projectidlist#" maxlength="35" list="true">)
			and		projects_projectloci.projectlociidfk = projectloci.id
		</cfquery>

		<cfreturn data>
	</cffunction>

	<cffunction name="getProjects" access="public" returnType="query" output="false"
				hint="Gets all the projects.">		
		
		<cfset var data = "">
		
		<cfquery name="data" datasource="#variables.dsn#">
			select 		projects.id, projects.name, count(issues.id) as numissues
			<cfif application.dbtype eq "oracle">	
			from		projects,
						issues
			Where		projects.id = issues.projectidfk(+)
			<cfelse>
			from		projects left join issues on projects.id = issues.projectidfk
			</cfif>
			group by	projects.id, projects.name
		</cfquery>

		<cfreturn data>
	</cffunction>

	<cffunction name="getSubscribedUsersForProject" access="public" returnType="query" output="false"
				hint="Gets all the email addresses for users subscribed to a project.">
		<cfargument name="id" type="uuid" required="true" hint="Project ID">
		<cfset var data = "">
		
		<cfquery name="data" datasource="#variables.dsn#">
			select	users.emailaddress
			from	projects_users_email, users
			where	projects_users_email.projectidfk = <cfqueryparam cfsqltype="cf_sql_varchar" value="#arguments.id#" maxlength="35">
			and		projects_users_email.useridfk = users.id
		</cfquery>
				
		<cfreturn data>
	</cffunction>
	
	<cffunction name="getUsersForProject" access="public" returnType="query" output="false"
				hint="Gets users for one project.">		
		<cfargument name="id" type="uuid" required="true" hint="Project ID">
		<cfset var data = "">
		
		<cfquery name="data" datasource="#variables.dsn#">
			select	users.id, users.username, users.name
			from	projects_users, users
			where	projects_users.projectidfk = <cfqueryparam cfsqltype="cf_sql_varchar" value="#arguments.id#" maxlength="35">
			and		projects_users.useridfk = users.id
		</cfquery>

		<cfreturn data>
	</cffunction>
	
	<cffunction name="getProjectsForUser" access="public" returnType="query" output="false"
				hint="Gets all the projects for the current user.">		
		<cfargument name="username" type="string" required="false" default="#getAuthUser()#">
		<cfset var data = "">

		<cfquery name="data" datasource="#variables.dsn#">
			select 		projects.id, projects.name
			from		projects 
			, projects_users, users
			where projects.id = projects_users.projectidfk
			and   projects_users.useridfk = users.id
			and	  users.username = <cfqueryparam cfsqltype="cf_sql_varchar" value="#arguments.username#" maxlength="35">
			order by projects.name asc
		</cfquery>

		<cfreturn data>
	</cffunction>

	<cffunction name="setEmailProjectsForUser" access="public" returnType="void" output="false"
				hint="Subscribes the user to a set of projects. Calls clear() first.">
		<cfargument name="userid" type="uuid" required="true">		
		<cfargument name="emailprojects" type="string" required="true">		
		<cfset var p = "">
		
		<cfset clearEmailProjectsForUser(arguments.userid)>
		
		<cfloop index="p" list="#arguments.emailprojects#">
			<cfquery datasource="#variables.dsn#">
			insert into projects_users_email(projectidfk, useridfk)
			values(<cfqueryparam cfsqltype="cf_sql_varchar" value="#p#" maxlength="35">,<cfqueryparam cfsqltype="cf_sql_varchar" value="#arguments.userid#" maxlength="35">)
			</cfquery>
		</cfloop>
			
	</cffunction>

	<cffunction name="setLociForProject" access="public" returnType="void" output="false"
				hint="Sets loci for  a project. Also clears current loci.">		
		<cfargument name="id" type="uuid" required="true" hint="Project ID">
		<cfargument name="loci" type="string" required="true" hint="List of UUIDs for Loci">
		
		<cfset var i = "">
		
		<cfset clearLociForProject(arguments.id)>
		
		<cfloop index="i" list="#arguments.loci#">
			<cfquery name="data" datasource="#variables.dsn#">
			insert into projects_projectloci(projectidfk, projectlociidfk)
			values(<cfqueryparam cfsqltype="cf_sql_varchar" value="#arguments.id#" maxlength="35">,<cfqueryparam cfsqltype="cf_sql_varchar" value="#i#" maxlength="35">)
			</cfquery>
		</cfloop>
		
	</cffunction>

	<cffunction name="setProjectsForUser" access="public" returnType="void" output="false"
				hint="Sets projects for a user. Also clears current project.">		
		<cfargument name="id" type="uuid" required="true" hint="User ID">
		<cfargument name="projects" type="string" required="true" hint="List of UUIDs for Projects">
		
		<cfset var i = "">
		
		<cfset clearProjectsForUser(arguments.id)>
		
		<cfloop index="i" list="#arguments.projects#">
			<cfquery name="data" datasource="#variables.dsn#">
			insert into projects_users(projectidfk, useridfk)
			values(<cfqueryparam cfsqltype="cf_sql_varchar" value="#i#" maxlength="35">,<cfqueryparam cfsqltype="cf_sql_varchar" value="#arguments.id#" maxlength="35">)
			</cfquery>
		</cfloop>
		
	</cffunction>

	<cffunction name="setUsersForProject" access="public" returnType="void" output="false"
				hint="Sets users for  a project. Also clears current users.">		
		<cfargument name="id" type="uuid" required="true" hint="Project ID">
		<cfargument name="users" type="string" required="true" hint="List of UUIDs for Users">
		
		<cfset var i = "">
		
		<cfset clearUsersForProject(arguments.id)>
		
		<cfloop index="i" list="#arguments.users#">
			<cfquery name="data" datasource="#variables.dsn#">
			insert into projects_users(projectidfk, useridfk)
			values(<cfqueryparam cfsqltype="cf_sql_varchar" value="#arguments.id#" maxlength="35">,<cfqueryparam cfsqltype="cf_sql_varchar" value="#i#" maxlength="35">)
			</cfquery>
		</cfloop>
		
	</cffunction>
			
</cfcomponent>